/*
The software in this package is distributed under the GNU General
Public License version 2 (with a special exception described below).

A copy of GNU General Public License (GPL) is included in this distribution,
in the file COPYING.GPL.

As a special exception, if other files instantiate templates or use macros
or inline functions from this file, or you compile this file and link it
with other works to produce a work based on this file, this file
does not by itself cause the resulting work to be covered
by the GNU General Public License.

However the source code for this file must still be made available
in accordance with section (3) of the GNU General Public License.

This exception does not invalidate any other reasons why a work based
on this file might be covered by the GNU General Public License.
*/
#include "config/configurationreader.h"

#include "boost_assert_handler.h"
#include "config/configurationcommandline.h"
#include "config/configurationfile.h"

using namespace std;
using boost::program_options::command_line_parser;
using boost::program_options::options_description;
using boost::program_options::parsed_options;
using boost::program_options::positional_options_description;
using boost::program_options::value;
using boost::program_options::variables_map;
using I2n::Logger::GlobalLogger;

//-----------------------------------------------------------------------------
// ConfigurationReader
//-----------------------------------------------------------------------------

/**
 * @brief Default constructor.
 */
ConfigurationReader::ConfigurationReader() :
    Config()
{
}

/**
 * @brief Destructor.
 */
ConfigurationReader::~ConfigurationReader()
{
}

/**
 * @brief Parses the command line and configuration file.
 *
 * @param argc The number of arguments in command line.
 * @param argv A vector containing the command line elements.
 *
 * @return @c true if the parsing was successful, otherwise returns @c false.
 */
bool ConfigurationReader::parse(
        const int argc,
        const char *argv[]
)
{
    BOOST_ASSERT( argc > 0 );
    BOOST_ASSERT( argv != NULL );

    variables_map vm;
    ConfigurationCommandLine command_line( argc, argv );
    bool command_line_processed = command_line.process( &vm );
    if ( command_line_processed )
    {
        bool command_line_parsed = command_line.parse( vm, &Config );
        if ( !command_line_parsed )
        {
            return false;
        }
    }

    string file_name = Config.get_config_file_name();
    if (file_name.empty())
    {
        return false;               // otherwise only get message about failed
        // file name assertion which is confusing because error lies elsewhere
    }

    ConfigurationFile file( file_name );
    bool configuration_file_processed = file.process( &vm );
    if ( configuration_file_processed )
    {
        if ( !file.parse( vm, &Config ) )
        {
            return false;
        }
    }

    bool input_processed = command_line_processed && configuration_file_processed;

    return input_processed;
}

/**
 * @return The object containing the configuration data parsed by this object.
 */
Configuration ConfigurationReader::get_configuration() const
{
    return Config;
}
